#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/pool.h"

#include <strings.h>
#include <stdlib.h>

void lm_pool_serve(lm_pool_t *pool, lm_mptp_t *packet, int sock, struct sockaddr *addr) {
  char ipaddr[INET6_ADDRSTRLEN];
  bzero(ipaddr, sizeof(ipaddr));
  sockaddr_to_str(addr, ipaddr);

  switch (MPTP_FLAGS_CODE(packet)) {

  // response PING with PONG
  case MPTP_C2S_PING:
    pdebug(__func__, "(%s) PING from %s returning PONG", pool->name, ipaddr);
    lm_mptp_init(packet, false, MPTP_S2C_PONG, true);
    lm_mptp_server_send(sock, packet, addr);
    break;

  // when INFO file is requested, send the file
  case MPTP_C2S_INFO:
    pdebug(__func__, "(%s) INFO from %s attempting to send info", pool->name, ipaddr);
    lm_mptp_sendfile(sock, addr, pool->paths.info);
    break;

  // when LIST file is requested, send the file
  case MPTP_C2S_LIST:
    pdebug(__func__, "(%s) LIST from %s attempting to send list", pool->name, ipaddr);
    lm_mptp_sendfile(sock, addr, pool->paths.list);
    break;

  }
}

void lm_pool_serve_thread(void *_arg) {
  lm_pool_thread_arg_t *arg = _arg;
  lm_pool_serve(arg->pool, &arg->packet, arg->sock, &arg->addr);
  free(arg);
}
